import { PluginUIContext } from '../../mol-plugin-ui/context';
import { Quat, Vec3 } from '../../mol-math/linear-algebra';
import { PluginCommands } from '../../mol-plugin/commands';
import { Camera } from '../../mol-canvas3d/camera';

let initialSnapshot: Camera.Snapshot;

export function storeCameraSnapshot(plugin: PluginUIContext): void {
    initialSnapshot = plugin.canvas3d!.camera.getSnapshot();
    console.log('initialSnapshot stored:', initialSnapshot);
}

export function loadInitialSnapshot(plugin: PluginUIContext): void {
    if (!initialSnapshot) {
        console.log('initialSnapshot is undefined');
    } else {
        console.log('initialSnapshot:', initialSnapshot);
        //PluginCommands.Camera.Reset(plugin, { snapshot: initialSnapshot, durationMs: 10 });
        requestAnimationFrame(() => plugin.canvas3d?.requestCameraReset({ snapshot: initialSnapshot }));
    }
}

export function rotateCamera(plugin: PluginUIContext): void {
    function rot90q(v: Vec3, axis: Vec3 = Vec3.create(1, 0, 0)): Vec3 {
        const q = Quat.setAxisAngle(Quat(), axis, -Math.PI/2);
        return Vec3.transformQuat(Vec3(), v, q);
    }
    function sub(v: Vec3, u: Vec3): Vec3 {
        return Vec3.sub(Vec3(), v, u);
    }
    function add(v: Vec3, u: Vec3): Vec3 {
        return Vec3.add(Vec3(), v, u);
    }


    const cam = plugin.canvas3d!.camera;
    const snapshot = cam.getSnapshot();
    const newSnapshot = {
        ...snapshot,
        // target + rotateBy90(postition - target)
        position: add(snapshot.target, rot90q(sub(snapshot.position, snapshot.target))),
        target: snapshot.target,
        up: Vec3.negUnitZ
    };

    PluginCommands.Camera.Reset(plugin, { snapshot: newSnapshot, durationMs: 1000 });
    //requestAnimationFrame(() => this.plugin.canvas3d?.requestCameraReset({ snapshot: newSnapshot }));
}
