import { mmCIF_Database } from "../../mol-io/reader/cif/schema/mmcif";
import { Mat4, Vec3 } from "../../mol-math/linear-algebra";
import { MmcifFormat } from "../../mol-model-formats/structure/mmcif";
import { Model } from "../../mol-model/structure";
import { AtomicConformation } from "../../mol-model/structure/model/properties/atomic";
import { PluginUIContext } from "../../mol-plugin-ui/context";
import { transformationForStateTransform } from "./transformation";
import { PDBTMTransformationMatrix } from "./types";

export namespace DebugUtil {
    let plugin: PluginUIContext;

    export function init(ctx: PluginUIContext) {
        plugin = ctx;
    }

    export function transformVector(v: Vec3, matrix: Mat4): Vec3 {
        const result = Vec3.transformMat4(Vec3(), v, matrix);
        console.log("transformVector: Input v & matrix:", v, Mat4.makeTable(matrix));
        console.log("transformVector: Result vector:", result);
        return result;
    }

    export function descriptorMxToMat4(matrix: PDBTMTransformationMatrix): Mat4 {
        console.log("descriptorMxToMat4: Input:", matrix);
        const result = transformationForStateTransform(matrix);
        console.log("descriptorMxToMat4: Result:", result);
        return result;
    }


    //
    // structure utils
    //


    export function getCellOfFirstStructure() {
        return plugin.managers.structure.hierarchy.current.models[0].structures[0].cell;
    }

    export function getModelOfFirstStructure() {
        return getCellOfFirstStructure().obj?.data.model;
    }

    // in case of mmCIF format
    export function dumpAtomProperties(authAtomId: number) {
        const model: Model|undefined = getModelOfFirstStructure();
        console.log("First model:", model);
        if(!model) {
            return;
        }

        const rowCount = model.atomicHierarchy.atoms._rowCount;
        const conformation: AtomicConformation = model.atomicConformation;
        const db: mmCIF_Database = (model.sourceData as MmcifFormat).data.db;
        const atom_site = db.atom_site;
        for (let index = 0; index < rowCount; index++) {
            if (conformation.atomId.value(index) == authAtomId) {
                console.log("Model Atom Conformation:", {
                    atomIdParameter: authAtomId,
                    atomId: conformation.atomId.value(index),
                    coords: [
                        conformation.x[index],
                        conformation.y[index],
                        conformation.z[index]
                    ],
                    xyzDefined: conformation.xyzDefined
                });

                console.log("Atom Source Data (mmCIF database):", {
                    authId: atom_site.auth_atom_id.value(index),
                    compId: atom_site.auth_comp_id.value(index),
                    authAsymId: atom_site.auth_asym_id.value(index),
                    authSeqId: atom_site.auth_seq_id.value(index),
                    coords: [
                        atom_site.Cartn_x.value(index),
                        atom_site.Cartn_y.value(index),
                        atom_site.Cartn_z.value(index)
                    ]
                });
            }
        }
    }
}
